
.helloworld-label {
    font-size: 72px;
    font-weight: bold;
    color: #ffffff;
    background-color: rgba(0,0,0,0.5);
    border-radius: 5px;
    padding: .5em;
}

.multimonitor-spacer {
	height: 4em;
}

.multimonitor-status-indicators-box {
    spacing: 0px;
}

.multimonitor-status-icon {
    padding: 0 2px;
}

.workspace-thumbnails-left {
  border-radius: 0 9px 9px 0; 
}

.workspace-thumbnails-left:rtl {
    border-radius: 9px 0 0 9px; 
}
