.just-perfection
{
    font-size: small;
}

.just-perfection .switcher-popup
{
    padding: 8px;
    spacing: 24px;
    font-size: small;
}

.just-perfection .switcher-list
{
    color: #eeeeec;
    background-color: rgba(0, 0, 0, .99);
    border: 0 solid rgba(255, 255, 255, .16);
    border-radius: 15px;
    padding: 12px;
    box-shadow: none;
}

.just-perfection .switcher-list .item-box
{
    padding: 8px;
    border-radius: 14px;
    border: 1px solid transparent;
    background-color: transparent;
}

.just-perfection .switcher-list .item-box:outlined
{
    border: 0;
    background-color: rgba(255, 255, 255, .1);
    box-shadow: inset 0 2px 2px 0 rgba(0, 0, 0, .4);
}

.just-perfection .switcher-list .item-box:selected
{
    background-color: #404040;
    color: #eeeeec;
}

.just-perfection .switcher-list .thumbnail-box
{
    padding: 2px;
    spacing: 6px;
}

.just-perfection .switcher-list .thumbnail
{
    width: 256px;
}

.just-perfection .switcher-list .separator
{
    width: 1px;
    background: #272727;
}

.just-perfection .switcher-list .switcher-list-item-container
{
    spacing: 12px;
}

.just-perfection .workspace-switcher-container
{
    background-color: rgba(0, 0, 0, .7);
    box-shadow: none;
    padding: 3px;
    border: 0;
    border-radius: 3px;
}

.just-perfection .workspace-switcher
{
    background: transparent;
    border: none;
    border-radius: 0;
    padding: 0;
    spacing: 4px;
}

.just-perfection .workspace-switcher
{
    background: rgba(0, 0, 0, .7);
    box-shadow: none;
    border: none;
    border-radius: 5px;
    padding: 0;
    spacing: 4px;
}

.just-perfection .ws-switcher-indicator,
.just-perfection .ws-switcher-indicator:active
{
    background: transparent;
    background-color: rgba(255, 255, 255, .02);
    height: 8px;
    width: 20px;
    border: 0;
    border-radius: 2px;
    padding: 2px;
    margin: 5px;
}

.just-perfection .ws-switcher-indicator:active
{
    background-color: rgba(255, 255, 255, .8);
}

.just-perfection .ws-switcher-box
{
    background: transparent;
    background-color: rgba(255, 255, 255, .02);
    height: 10px;
    background-size: 0;
    border: 0;
    border-radius: 1px;
}

.just-perfection .ws-switcher-active-up,
.just-perfection .ws-switcher-active-down,
.just-perfection .ws-switcher-active-left,
.just-perfection .ws-switcher-active-right
{
    height: 10px;
    background-color: rgba(255, 255, 255, .8);
    border: 0;
    border-radius: 1px;
}

.just-perfection .osd-window
{
    background-color: rgba(0, 0, 0, 1);
    border: 0;
    border-radius: 8px;
    padding: 10px;
    box-shadow: none;
}

.just-perfection .osd-window StIcon
{
    width: 22px;
    height: 22px;
}

.just-perfection .tile-preview
{
    background-color: rgba(255, 255, 255, 0.15);
    border: 1px solid rgba(255, 255, 255, 0.6);
    border-radius: 0;
}

.just-perfection-api-accent-color-icon StIcon
{
    color: -st-accent-color;
}

.just-perfection-api-accent-color-icon #panelActivities .workspace-dot
{
    background-color: -st-accent-color;
}

.just-perfection-api-accent-color-icon .quick-toggle-menu .icon.active,
.just-perfection-api-accent-color-icon .quick-toggle:checked StIcon,
.just-perfection-api-accent-color-icon .quick-menu-toggle .quick-toggle-arrow:checked StIcon
{
    color: -st-accent-fg-color;
}

.just-perfection-api-no-search #overview
{
    spacing: 0;
}

.just-perfection-api-no-search .overview-controls
{
    padding-top: 36px;
    padding-bottom: 36px;
}

/* commented because the icon will be shown when the window is getting dragged */
/* icon-dropshadow isn't specific to the window picker icon but that's a quick fix for now */
/* .just-perfection-api-no-window-picker-icon .workspaces-view StIcon */
.just-perfection-api-no-window-picker-icon .icon-dropshadow
{
    width: 0;
    height: 0;
    padding: 0;
}

.just-perfection-api-bottom-panel #overview
{
    margin-bottom: 24px;
}

.just-perfection-api-bottom-panel .popup-menu.panel-menu
{
    margin-bottom: 0;
}

.just-perfection-api-bottom-panel.just-perfection-api-no-panel #overview
{
    margin-bottom: 0;
}

.just-perfection-api-no-panel #panel
{
    background-color: transparent;
}

.just-perfection-api-no-panel-arrow .panel-button .popup-menu-arrow
{
    width: 0;
    height: 0;
}

.just-perfection-api-no-panel-notification-icon .clock-display StIcon
{
    width: 0;
    height: 0;
    padding: 0;
}

.just-perfection-api-no-panel-notification-icon .clock-display-box
{
    padding: 0;
    spacing: 0;
}

.just-perfection-api-no-show-apps-button .show-apps,
.just-perfection-api-no-show-apps-button .show-apps StIcon
{
    width: 0;
    margin: 0;
    spacing: 0;
}

.just-perfection-api-dash-icon-size16 #dash StIcon { height: 16px; width: 16px; }
.just-perfection-api-dash-icon-size22 #dash StIcon { height: 22px; width: 22px; }
.just-perfection-api-dash-icon-size24 #dash StIcon { height: 24px; width: 24px; }
.just-perfection-api-dash-icon-size32 #dash StIcon { height: 32px; width: 32px; }
.just-perfection-api-dash-icon-size40 #dash StIcon { height: 40px; width: 40px; }
.just-perfection-api-dash-icon-size48 #dash StIcon { height: 48px; width: 48px; }
.just-perfection-api-dash-icon-size56 #dash StIcon { height: 56px; width: 56px; }
.just-perfection-api-dash-icon-size64 #dash StIcon { height: 64px; width: 64px; }


.just-perfection-api-panel-button-padding-size0 .panel-button { -natural-hpadding: 0px; -minimum-hpadding: 0px; }
.just-perfection-api-panel-button-padding-size1 .panel-button { -natural-hpadding: 1px; -minimum-hpadding: 1px; }
.just-perfection-api-panel-button-padding-size2 .panel-button { -natural-hpadding: 2px; -minimum-hpadding: 2px; }
.just-perfection-api-panel-button-padding-size3 .panel-button { -natural-hpadding: 3px; -minimum-hpadding: 3px; }
.just-perfection-api-panel-button-padding-size4 .panel-button { -natural-hpadding: 4px; -minimum-hpadding: 4px; }
.just-perfection-api-panel-button-padding-size5 .panel-button { -natural-hpadding: 5px; -minimum-hpadding: 5px; }
.just-perfection-api-panel-button-padding-size6 .panel-button { -natural-hpadding: 6px; -minimum-hpadding: 6px; }
.just-perfection-api-panel-button-padding-size7 .panel-button { -natural-hpadding: 7px; -minimum-hpadding: 7px; }
.just-perfection-api-panel-button-padding-size8 .panel-button { -natural-hpadding: 8px; -minimum-hpadding: 8px; }
.just-perfection-api-panel-button-padding-size9 .panel-button { -natural-hpadding: 9px; -minimum-hpadding: 9px; }
.just-perfection-api-panel-button-padding-size10 .panel-button { -natural-hpadding: 10px; -minimum-hpadding: 10px; }
.just-perfection-api-panel-button-padding-size11 .panel-button { -natural-hpadding: 11px; -minimum-hpadding: 11px; }
.just-perfection-api-panel-button-padding-size12 .panel-button { -natural-hpadding: 12px; -minimum-hpadding: 12px; }
.just-perfection-api-panel-button-padding-size13 .panel-button { -natural-hpadding: 13px; -minimum-hpadding: 13px; }
.just-perfection-api-panel-button-padding-size14 .panel-button { -natural-hpadding: 14px; -minimum-hpadding: 14px; }
.just-perfection-api-panel-button-padding-size15 .panel-button { -natural-hpadding: 15px; -minimum-hpadding: 15px; }
.just-perfection-api-panel-button-padding-size16 .panel-button { -natural-hpadding: 16px; -minimum-hpadding: 16px; }
.just-perfection-api-panel-button-padding-size17 .panel-button { -natural-hpadding: 17px; -minimum-hpadding: 17px; }
.just-perfection-api-panel-button-padding-size18 .panel-button { -natural-hpadding: 18px; -minimum-hpadding: 18px; }
.just-perfection-api-panel-button-padding-size19 .panel-button { -natural-hpadding: 19px; -minimum-hpadding: 19px; }
.just-perfection-api-panel-button-padding-size20 .panel-button { -natural-hpadding: 20px; -minimum-hpadding: 20px; }
.just-perfection-api-panel-button-padding-size21 .panel-button { -natural-hpadding: 21px; -minimum-hpadding: 21px; }
.just-perfection-api-panel-button-padding-size22 .panel-button { -natural-hpadding: 22px; -minimum-hpadding: 22px; }
.just-perfection-api-panel-button-padding-size23 .panel-button { -natural-hpadding: 23px; -minimum-hpadding: 23px; }
.just-perfection-api-panel-button-padding-size24 .panel-button { -natural-hpadding: 24px; -minimum-hpadding: 24px; }
.just-perfection-api-panel-button-padding-size25 .panel-button { -natural-hpadding: 25px; -minimum-hpadding: 25px; }
.just-perfection-api-panel-button-padding-size26 .panel-button { -natural-hpadding: 26px; -minimum-hpadding: 26px; }
.just-perfection-api-panel-button-padding-size27 .panel-button { -natural-hpadding: 27px; -minimum-hpadding: 27px; }
.just-perfection-api-panel-button-padding-size28 .panel-button { -natural-hpadding: 28px; -minimum-hpadding: 28px; }
.just-perfection-api-panel-button-padding-size29 .panel-button { -natural-hpadding: 29px; -minimum-hpadding: 29px; }
.just-perfection-api-panel-button-padding-size30 .panel-button { -natural-hpadding: 30px; -minimum-hpadding: 30px; }
.just-perfection-api-panel-button-padding-size31 .panel-button { -natural-hpadding: 31px; -minimum-hpadding: 31px; }
.just-perfection-api-panel-button-padding-size32 .panel-button { -natural-hpadding: 32px; -minimum-hpadding: 32px; }
.just-perfection-api-panel-button-padding-size33 .panel-button { -natural-hpadding: 33px; -minimum-hpadding: 33px; }
.just-perfection-api-panel-button-padding-size34 .panel-button { -natural-hpadding: 34px; -minimum-hpadding: 34px; }
.just-perfection-api-panel-button-padding-size35 .panel-button { -natural-hpadding: 35px; -minimum-hpadding: 35px; }
.just-perfection-api-panel-button-padding-size36 .panel-button { -natural-hpadding: 36px; -minimum-hpadding: 36px; }
.just-perfection-api-panel-button-padding-size37 .panel-button { -natural-hpadding: 37px; -minimum-hpadding: 37px; }
.just-perfection-api-panel-button-padding-size38 .panel-button { -natural-hpadding: 38px; -minimum-hpadding: 38px; }
.just-perfection-api-panel-button-padding-size39 .panel-button { -natural-hpadding: 39px; -minimum-hpadding: 39px; }
.just-perfection-api-panel-button-padding-size40 .panel-button { -natural-hpadding: 40px; -minimum-hpadding: 40px; }
.just-perfection-api-panel-button-padding-size41 .panel-button { -natural-hpadding: 41px; -minimum-hpadding: 41px; }
.just-perfection-api-panel-button-padding-size42 .panel-button { -natural-hpadding: 42px; -minimum-hpadding: 42px; }
.just-perfection-api-panel-button-padding-size43 .panel-button { -natural-hpadding: 43px; -minimum-hpadding: 43px; }
.just-perfection-api-panel-button-padding-size44 .panel-button { -natural-hpadding: 44px; -minimum-hpadding: 44px; }
.just-perfection-api-panel-button-padding-size45 .panel-button { -natural-hpadding: 45px; -minimum-hpadding: 45px; }
.just-perfection-api-panel-button-padding-size46 .panel-button { -natural-hpadding: 46px; -minimum-hpadding: 46px; }
.just-perfection-api-panel-button-padding-size47 .panel-button { -natural-hpadding: 47px; -minimum-hpadding: 47px; }
.just-perfection-api-panel-button-padding-size48 .panel-button { -natural-hpadding: 48px; -minimum-hpadding: 48px; }
.just-perfection-api-panel-button-padding-size49 .panel-button { -natural-hpadding: 49px; -minimum-hpadding: 49px; }
.just-perfection-api-panel-button-padding-size50 .panel-button { -natural-hpadding: 50px; -minimum-hpadding: 50px; }
.just-perfection-api-panel-button-padding-size51 .panel-button { -natural-hpadding: 51px; -minimum-hpadding: 51px; }
.just-perfection-api-panel-button-padding-size52 .panel-button { -natural-hpadding: 52px; -minimum-hpadding: 52px; }
.just-perfection-api-panel-button-padding-size53 .panel-button { -natural-hpadding: 53px; -minimum-hpadding: 53px; }
.just-perfection-api-panel-button-padding-size54 .panel-button { -natural-hpadding: 54px; -minimum-hpadding: 54px; }
.just-perfection-api-panel-button-padding-size55 .panel-button { -natural-hpadding: 55px; -minimum-hpadding: 55px; }
.just-perfection-api-panel-button-padding-size56 .panel-button { -natural-hpadding: 56px; -minimum-hpadding: 56px; }
.just-perfection-api-panel-button-padding-size57 .panel-button { -natural-hpadding: 57px; -minimum-hpadding: 57px; }
.just-perfection-api-panel-button-padding-size58 .panel-button { -natural-hpadding: 58px; -minimum-hpadding: 58px; }
.just-perfection-api-panel-button-padding-size59 .panel-button { -natural-hpadding: 59px; -minimum-hpadding: 59px; }
.just-perfection-api-panel-button-padding-size60 .panel-button { -natural-hpadding: 60px; -minimum-hpadding: 60px; }

.just-perfection-api-no-window-caption .window-caption
{
    spacing: 0;
    background-color: transparent;
    border-radius: 0;
    border: 0;
    padding: 0;
    font-size: 0;
}

.just-perfection-api-workspace-background-radius-size0 .workspace-background { border-radius: 0; }
.just-perfection-api-workspace-background-radius-size1 .workspace-background { border-radius: 1px; }
.just-perfection-api-workspace-background-radius-size2 .workspace-background { border-radius: 2px; }
.just-perfection-api-workspace-background-radius-size3 .workspace-background { border-radius: 3px; }
.just-perfection-api-workspace-background-radius-size4 .workspace-background { border-radius: 4px; }
.just-perfection-api-workspace-background-radius-size5 .workspace-background { border-radius: 5px; }
.just-perfection-api-workspace-background-radius-size6 .workspace-background { border-radius: 6px; }
.just-perfection-api-workspace-background-radius-size7 .workspace-background { border-radius: 7px; }
.just-perfection-api-workspace-background-radius-size8 .workspace-background { border-radius: 8px; }
.just-perfection-api-workspace-background-radius-size9 .workspace-background { border-radius: 9px; }
.just-perfection-api-workspace-background-radius-size10 .workspace-background { border-radius: 10px; }
.just-perfection-api-workspace-background-radius-size11 .workspace-background { border-radius: 11px; }
.just-perfection-api-workspace-background-radius-size12 .workspace-background { border-radius: 12px; }
.just-perfection-api-workspace-background-radius-size13 .workspace-background { border-radius: 13px; }
.just-perfection-api-workspace-background-radius-size14 .workspace-background { border-radius: 14px; }
.just-perfection-api-workspace-background-radius-size15 .workspace-background { border-radius: 15px; }
.just-perfection-api-workspace-background-radius-size16 .workspace-background { border-radius: 16px; }
.just-perfection-api-workspace-background-radius-size17 .workspace-background { border-radius: 17px; }
.just-perfection-api-workspace-background-radius-size18 .workspace-background { border-radius: 18px; }
.just-perfection-api-workspace-background-radius-size19 .workspace-background { border-radius: 19px; }
.just-perfection-api-workspace-background-radius-size20 .workspace-background { border-radius: 20px; }
.just-perfection-api-workspace-background-radius-size21 .workspace-background { border-radius: 21px; }
.just-perfection-api-workspace-background-radius-size22 .workspace-background { border-radius: 22px; }
.just-perfection-api-workspace-background-radius-size23 .workspace-background { border-radius: 23px; }
.just-perfection-api-workspace-background-radius-size24 .workspace-background { border-radius: 24px; }
.just-perfection-api-workspace-background-radius-size25 .workspace-background { border-radius: 25px; }
.just-perfection-api-workspace-background-radius-size26 .workspace-background { border-radius: 26px; }
.just-perfection-api-workspace-background-radius-size27 .workspace-background { border-radius: 27px; }
.just-perfection-api-workspace-background-radius-size28 .workspace-background { border-radius: 28px; }
.just-perfection-api-workspace-background-radius-size29 .workspace-background { border-radius: 29px; }
.just-perfection-api-workspace-background-radius-size30 .workspace-background { border-radius: 30px; }
.just-perfection-api-workspace-background-radius-size31 .workspace-background { border-radius: 31px; }
.just-perfection-api-workspace-background-radius-size32 .workspace-background { border-radius: 32px; }
.just-perfection-api-workspace-background-radius-size33 .workspace-background { border-radius: 33px; }
.just-perfection-api-workspace-background-radius-size34 .workspace-background { border-radius: 34px; }
.just-perfection-api-workspace-background-radius-size35 .workspace-background { border-radius: 35px; }
.just-perfection-api-workspace-background-radius-size36 .workspace-background { border-radius: 36px; }
.just-perfection-api-workspace-background-radius-size37 .workspace-background { border-radius: 37px; }
.just-perfection-api-workspace-background-radius-size38 .workspace-background { border-radius: 38px; }
.just-perfection-api-workspace-background-radius-size39 .workspace-background { border-radius: 39px; }
.just-perfection-api-workspace-background-radius-size40 .workspace-background { border-radius: 40px; }
.just-perfection-api-workspace-background-radius-size41 .workspace-background { border-radius: 41px; }
.just-perfection-api-workspace-background-radius-size42 .workspace-background { border-radius: 42px; }
.just-perfection-api-workspace-background-radius-size43 .workspace-background { border-radius: 43px; }
.just-perfection-api-workspace-background-radius-size44 .workspace-background { border-radius: 44px; }
.just-perfection-api-workspace-background-radius-size45 .workspace-background { border-radius: 45px; }
.just-perfection-api-workspace-background-radius-size46 .workspace-background { border-radius: 46px; }
.just-perfection-api-workspace-background-radius-size47 .workspace-background { border-radius: 47px; }
.just-perfection-api-workspace-background-radius-size48 .workspace-background { border-radius: 48px; }
.just-perfection-api-workspace-background-radius-size49 .workspace-background { border-radius: 49px; }
.just-perfection-api-workspace-background-radius-size50 .workspace-background { border-radius: 50px; }
.just-perfection-api-workspace-background-radius-size51 .workspace-background { border-radius: 51px; }
.just-perfection-api-workspace-background-radius-size52 .workspace-background { border-radius: 52px; }
.just-perfection-api-workspace-background-radius-size53 .workspace-background { border-radius: 53px; }
.just-perfection-api-workspace-background-radius-size54 .workspace-background { border-radius: 54px; }
.just-perfection-api-workspace-background-radius-size55 .workspace-background { border-radius: 55px; }
.just-perfection-api-workspace-background-radius-size56 .workspace-background { border-radius: 56px; }
.just-perfection-api-workspace-background-radius-size57 .workspace-background { border-radius: 57px; }
.just-perfection-api-workspace-background-radius-size58 .workspace-background { border-radius: 58px; }
.just-perfection-api-workspace-background-radius-size59 .workspace-background { border-radius: 59px; }
.just-perfection-api-workspace-background-radius-size60 .workspace-background { border-radius: 60px; }

.just-perfection-api-panel-indicator-padding-size0 .panel-status-indicators-box { spacing: 0; }
.just-perfection-api-panel-indicator-padding-size1 .panel-status-indicators-box { spacing: 1px; }
.just-perfection-api-panel-indicator-padding-size2 .panel-status-indicators-box { spacing: 2px; }
.just-perfection-api-panel-indicator-padding-size3 .panel-status-indicators-box { spacing: 3px; }
.just-perfection-api-panel-indicator-padding-size4 .panel-status-indicators-box { spacing: 4px; }
.just-perfection-api-panel-indicator-padding-size5 .panel-status-indicators-box { spacing: 5px; }
.just-perfection-api-panel-indicator-padding-size6 .panel-status-indicators-box { spacing: 6px; }
.just-perfection-api-panel-indicator-padding-size7 .panel-status-indicators-box { spacing: 7px; }
.just-perfection-api-panel-indicator-padding-size8 .panel-status-indicators-box { spacing: 8px; }
.just-perfection-api-panel-indicator-padding-size9 .panel-status-indicators-box { spacing: 9px; }
.just-perfection-api-panel-indicator-padding-size10 .panel-status-indicators-box { spacing: 10px; }
.just-perfection-api-panel-indicator-padding-size11 .panel-status-indicators-box { spacing: 11px; }
.just-perfection-api-panel-indicator-padding-size12 .panel-status-indicators-box { spacing: 12px; }
.just-perfection-api-panel-indicator-padding-size13 .panel-status-indicators-box { spacing: 13px; }
.just-perfection-api-panel-indicator-padding-size14 .panel-status-indicators-box { spacing: 14px; }
.just-perfection-api-panel-indicator-padding-size15 .panel-status-indicators-box { spacing: 15px; }
.just-perfection-api-panel-indicator-padding-size16 .panel-status-indicators-box { spacing: 16px; }
.just-perfection-api-panel-indicator-padding-size17 .panel-status-indicators-box { spacing: 17px; }
.just-perfection-api-panel-indicator-padding-size18 .panel-status-indicators-box { spacing: 18px; }
.just-perfection-api-panel-indicator-padding-size19 .panel-status-indicators-box { spacing: 19px; }
.just-perfection-api-panel-indicator-padding-size20 .panel-status-indicators-box { spacing: 20px; }
.just-perfection-api-panel-indicator-padding-size21 .panel-status-indicators-box { spacing: 21px; }
.just-perfection-api-panel-indicator-padding-size22 .panel-status-indicators-box { spacing: 22px; }
.just-perfection-api-panel-indicator-padding-size23 .panel-status-indicators-box { spacing: 23px; }
.just-perfection-api-panel-indicator-padding-size24 .panel-status-indicators-box { spacing: 24px; }
.just-perfection-api-panel-indicator-padding-size25 .panel-status-indicators-box { spacing: 25px; }
.just-perfection-api-panel-indicator-padding-size26 .panel-status-indicators-box { spacing: 26px; }
.just-perfection-api-panel-indicator-padding-size27 .panel-status-indicators-box { spacing: 27px; }
.just-perfection-api-panel-indicator-padding-size28 .panel-status-indicators-box { spacing: 28px; }
.just-perfection-api-panel-indicator-padding-size29 .panel-status-indicators-box { spacing: 29px; }
.just-perfection-api-panel-indicator-padding-size30 .panel-status-indicators-box { spacing: 30px; }
.just-perfection-api-panel-indicator-padding-size31 .panel-status-indicators-box { spacing: 31px; }
.just-perfection-api-panel-indicator-padding-size32 .panel-status-indicators-box { spacing: 32px; }
.just-perfection-api-panel-indicator-padding-size33 .panel-status-indicators-box { spacing: 33px; }
.just-perfection-api-panel-indicator-padding-size34 .panel-status-indicators-box { spacing: 34px; }
.just-perfection-api-panel-indicator-padding-size35 .panel-status-indicators-box { spacing: 35px; }
.just-perfection-api-panel-indicator-padding-size36 .panel-status-indicators-box { spacing: 36px; }
.just-perfection-api-panel-indicator-padding-size37 .panel-status-indicators-box { spacing: 37px; }
.just-perfection-api-panel-indicator-padding-size38 .panel-status-indicators-box { spacing: 38px; }
.just-perfection-api-panel-indicator-padding-size39 .panel-status-indicators-box { spacing: 39px; }
.just-perfection-api-panel-indicator-padding-size40 .panel-status-indicators-box { spacing: 40px; }
.just-perfection-api-panel-indicator-padding-size41 .panel-status-indicators-box { spacing: 41px; }
.just-perfection-api-panel-indicator-padding-size42 .panel-status-indicators-box { spacing: 42px; }
.just-perfection-api-panel-indicator-padding-size43 .panel-status-indicators-box { spacing: 43px; }
.just-perfection-api-panel-indicator-padding-size44 .panel-status-indicators-box { spacing: 44px; }
.just-perfection-api-panel-indicator-padding-size45 .panel-status-indicators-box { spacing: 45px; }
.just-perfection-api-panel-indicator-padding-size46 .panel-status-indicators-box { spacing: 46px; }
.just-perfection-api-panel-indicator-padding-size47 .panel-status-indicators-box { spacing: 47px; }
.just-perfection-api-panel-indicator-padding-size48 .panel-status-indicators-box { spacing: 48px; }
.just-perfection-api-panel-indicator-padding-size49 .panel-status-indicators-box { spacing: 49px; }
.just-perfection-api-panel-indicator-padding-size50 .panel-status-indicators-box { spacing: 50px; }
.just-perfection-api-panel-indicator-padding-size51 .panel-status-indicators-box { spacing: 51px; }
.just-perfection-api-panel-indicator-padding-size52 .panel-status-indicators-box { spacing: 52px; }
.just-perfection-api-panel-indicator-padding-size53 .panel-status-indicators-box { spacing: 53px; }
.just-perfection-api-panel-indicator-padding-size54 .panel-status-indicators-box { spacing: 54px; }
.just-perfection-api-panel-indicator-padding-size55 .panel-status-indicators-box { spacing: 55px; }
.just-perfection-api-panel-indicator-padding-size56 .panel-status-indicators-box { spacing: 56px; }
.just-perfection-api-panel-indicator-padding-size57 .panel-status-indicators-box { spacing: 57px; }
.just-perfection-api-panel-indicator-padding-size58 .panel-status-indicators-box { spacing: 58px; }
.just-perfection-api-panel-indicator-padding-size59 .panel-status-indicators-box { spacing: 59px; }
.just-perfection-api-panel-indicator-padding-size60 .panel-status-indicators-box { spacing: 60px; }

.just-perfection-api-no-window-close .window-close,
.just-perfection-api-no-window-close.just-perfection .window-close
{
    height: 0;
    width: 30px;
    background: transparent;
    border-radius: 0;
    border: 0;
    spacing: 0;
    box-shadow: none;
}

.just-perfection-api-no-ripple-box .ripple-box
{
    background-color: transparent;
}

/**
 * This can refresh other styles that is not inculding any standard css styles.
 * values are not important here. Just do something that can change the look
 */
.just-perfection-api-refresh-styles .panel-button,
.just-perfection-api-refresh-styles .panel-status-indicators-box,
.just-perfection-api-refresh-styles .panel-button StIcon,
.just-perfection-api-refresh-styles .system-status-icon
{
    background-color: gold;
    color: gold;
    border-radius: 100px;
}

.just-perfection-api-no-weather .weather-button,
.just-perfection-api-no-weather .weather-button *,
.just-perfection-api-no-events-button .events-button,
.just-perfection-api-no-events-button .events-button *,
.just-perfection-api-no-screen-sharing-indicator .screen-sharing-indicator,
.just-perfection-api-no-screen-sharing-indicator .screen-sharing-indicator *,
.just-perfection-api-no-screen-recording-indicator .screen-recording-indicator,
.just-perfection-api-no-screen-recording-indicator .screen-recording-indicator *
{
    padding: 0;
    margin: 0;
    border: 0;
    height: 0;
    width: 0;
}

.just-perfection-api-panel-icon-size1 .system-status-icon, .just-perfection-api-panel-icon-size1 .panel-button StIcon { icon-size: 1px; }
.just-perfection-api-panel-icon-size2 .system-status-icon, .just-perfection-api-panel-icon-size2 .panel-button StIcon { icon-size: 2px; }
.just-perfection-api-panel-icon-size3 .system-status-icon, .just-perfection-api-panel-icon-size3 .panel-button StIcon { icon-size: 3px; }
.just-perfection-api-panel-icon-size4 .system-status-icon, .just-perfection-api-panel-icon-size4 .panel-button StIcon { icon-size: 4px; }
.just-perfection-api-panel-icon-size5 .system-status-icon, .just-perfection-api-panel-icon-size5 .panel-button StIcon { icon-size: 5px; }
.just-perfection-api-panel-icon-size6 .system-status-icon, .just-perfection-api-panel-icon-size6 .panel-button StIcon { icon-size: 6px; }
.just-perfection-api-panel-icon-size7 .system-status-icon, .just-perfection-api-panel-icon-size7 .panel-button StIcon { icon-size: 7px; }
.just-perfection-api-panel-icon-size8 .system-status-icon, .just-perfection-api-panel-icon-size8 .panel-button StIcon { icon-size: 8px; }
.just-perfection-api-panel-icon-size9 .system-status-icon, .just-perfection-api-panel-icon-size9 .panel-button StIcon { icon-size: 9px; }
.just-perfection-api-panel-icon-size10 .system-status-icon, .just-perfection-api-panel-icon-size10 .panel-button StIcon { icon-size: 10px; }
.just-perfection-api-panel-icon-size11 .system-status-icon, .just-perfection-api-panel-icon-size11 .panel-button StIcon { icon-size: 11px; }
.just-perfection-api-panel-icon-size12 .system-status-icon, .just-perfection-api-panel-icon-size12 .panel-button StIcon { icon-size: 12px; }
.just-perfection-api-panel-icon-size13 .system-status-icon, .just-perfection-api-panel-icon-size13 .panel-button StIcon { icon-size: 13px; }
.just-perfection-api-panel-icon-size14 .system-status-icon, .just-perfection-api-panel-icon-size14 .panel-button StIcon { icon-size: 14px; }
.just-perfection-api-panel-icon-size15 .system-status-icon, .just-perfection-api-panel-icon-size15 .panel-button StIcon { icon-size: 15px; }
.just-perfection-api-panel-icon-size16 .system-status-icon, .just-perfection-api-panel-icon-size16 .panel-button StIcon { icon-size: 16px; }
.just-perfection-api-panel-icon-size17 .system-status-icon, .just-perfection-api-panel-icon-size17 .panel-button StIcon { icon-size: 17px; }
.just-perfection-api-panel-icon-size18 .system-status-icon, .just-perfection-api-panel-icon-size18 .panel-button StIcon { icon-size: 18px; }
.just-perfection-api-panel-icon-size19 .system-status-icon, .just-perfection-api-panel-icon-size19 .panel-button StIcon { icon-size: 19px; }
.just-perfection-api-panel-icon-size20 .system-status-icon, .just-perfection-api-panel-icon-size20 .panel-button StIcon { icon-size: 20px; }
.just-perfection-api-panel-icon-size21 .system-status-icon, .just-perfection-api-panel-icon-size21 .panel-button StIcon { icon-size: 21px; }
.just-perfection-api-panel-icon-size22 .system-status-icon, .just-perfection-api-panel-icon-size22 .panel-button StIcon { icon-size: 22px; }
.just-perfection-api-panel-icon-size23 .system-status-icon, .just-perfection-api-panel-icon-size23 .panel-button StIcon { icon-size: 23px; }
.just-perfection-api-panel-icon-size24 .system-status-icon, .just-perfection-api-panel-icon-size24 .panel-button StIcon { icon-size: 24px; }
.just-perfection-api-panel-icon-size25 .system-status-icon, .just-perfection-api-panel-icon-size25 .panel-button StIcon { icon-size: 25px; }
.just-perfection-api-panel-icon-size26 .system-status-icon, .just-perfection-api-panel-icon-size26 .panel-button StIcon { icon-size: 26px; }
.just-perfection-api-panel-icon-size27 .system-status-icon, .just-perfection-api-panel-icon-size27 .panel-button StIcon { icon-size: 27px; }
.just-perfection-api-panel-icon-size28 .system-status-icon, .just-perfection-api-panel-icon-size28 .panel-button StIcon { icon-size: 28px; }
.just-perfection-api-panel-icon-size29 .system-status-icon, .just-perfection-api-panel-icon-size29 .panel-button StIcon { icon-size: 29px; }
.just-perfection-api-panel-icon-size30 .system-status-icon, .just-perfection-api-panel-icon-size30 .panel-button StIcon { icon-size: 30px; }
.just-perfection-api-panel-icon-size31 .system-status-icon, .just-perfection-api-panel-icon-size31 .panel-button StIcon { icon-size: 31px; }
.just-perfection-api-panel-icon-size32 .system-status-icon, .just-perfection-api-panel-icon-size32 .panel-button StIcon { icon-size: 32px; }
.just-perfection-api-panel-icon-size33 .system-status-icon, .just-perfection-api-panel-icon-size33 .panel-button StIcon { icon-size: 33px; }
.just-perfection-api-panel-icon-size34 .system-status-icon, .just-perfection-api-panel-icon-size34 .panel-button StIcon { icon-size: 34px; }
.just-perfection-api-panel-icon-size35 .system-status-icon, .just-perfection-api-panel-icon-size35 .panel-button StIcon { icon-size: 35px; }
.just-perfection-api-panel-icon-size36 .system-status-icon, .just-perfection-api-panel-icon-size36 .panel-button StIcon { icon-size: 36px; }
.just-perfection-api-panel-icon-size37 .system-status-icon, .just-perfection-api-panel-icon-size37 .panel-button StIcon { icon-size: 37px; }
.just-perfection-api-panel-icon-size38 .system-status-icon, .just-perfection-api-panel-icon-size38 .panel-button StIcon { icon-size: 38px; }
.just-perfection-api-panel-icon-size39 .system-status-icon, .just-perfection-api-panel-icon-size39 .panel-button StIcon { icon-size: 39px; }
.just-perfection-api-panel-icon-size40 .system-status-icon, .just-perfection-api-panel-icon-size40 .panel-button StIcon { icon-size: 40px; }
.just-perfection-api-panel-icon-size41 .system-status-icon, .just-perfection-api-panel-icon-size41 .panel-button StIcon { icon-size: 41px; }
.just-perfection-api-panel-icon-size42 .system-status-icon, .just-perfection-api-panel-icon-size42 .panel-button StIcon { icon-size: 42px; }
.just-perfection-api-panel-icon-size43 .system-status-icon, .just-perfection-api-panel-icon-size43 .panel-button StIcon { icon-size: 43px; }
.just-perfection-api-panel-icon-size44 .system-status-icon, .just-perfection-api-panel-icon-size44 .panel-button StIcon { icon-size: 44px; }
.just-perfection-api-panel-icon-size45 .system-status-icon, .just-perfection-api-panel-icon-size45 .panel-button StIcon { icon-size: 45px; }
.just-perfection-api-panel-icon-size46 .system-status-icon, .just-perfection-api-panel-icon-size46 .panel-button StIcon { icon-size: 46px; }
.just-perfection-api-panel-icon-size47 .system-status-icon, .just-perfection-api-panel-icon-size47 .panel-button StIcon { icon-size: 47px; }
.just-perfection-api-panel-icon-size48 .system-status-icon, .just-perfection-api-panel-icon-size48 .panel-button StIcon { icon-size: 48px; }
.just-perfection-api-panel-icon-size49 .system-status-icon, .just-perfection-api-panel-icon-size49 .panel-button StIcon { icon-size: 49px; }
.just-perfection-api-panel-icon-size50 .system-status-icon, .just-perfection-api-panel-icon-size50 .panel-button StIcon { icon-size: 50px; }
.just-perfection-api-panel-icon-size51 .system-status-icon, .just-perfection-api-panel-icon-size51 .panel-button StIcon { icon-size: 51px; }
.just-perfection-api-panel-icon-size52 .system-status-icon, .just-perfection-api-panel-icon-size52 .panel-button StIcon { icon-size: 52px; }
.just-perfection-api-panel-icon-size53 .system-status-icon, .just-perfection-api-panel-icon-size53 .panel-button StIcon { icon-size: 53px; }
.just-perfection-api-panel-icon-size54 .system-status-icon, .just-perfection-api-panel-icon-size54 .panel-button StIcon { icon-size: 54px; }
.just-perfection-api-panel-icon-size55 .system-status-icon, .just-perfection-api-panel-icon-size55 .panel-button StIcon { icon-size: 55px; }
.just-perfection-api-panel-icon-size56 .system-status-icon, .just-perfection-api-panel-icon-size56 .panel-button StIcon { icon-size: 56px; }
.just-perfection-api-panel-icon-size57 .system-status-icon, .just-perfection-api-panel-icon-size57 .panel-button StIcon { icon-size: 57px; }
.just-perfection-api-panel-icon-size58 .system-status-icon, .just-perfection-api-panel-icon-size58 .panel-button StIcon { icon-size: 58px; }
.just-perfection-api-panel-icon-size59 .system-status-icon, .just-perfection-api-panel-icon-size59 .panel-button StIcon { icon-size: 59px; }
.just-perfection-api-panel-icon-size60 .system-status-icon, .just-perfection-api-panel-icon-size60 .panel-button StIcon { icon-size: 60px; }

.just-perfection-api-controls-manager-spacing-size0 .controls-manager, .just-perfection-api-controls-manager-spacing-size0 .secondary-monitor-workspaces { spacing: 0; }
.just-perfection-api-controls-manager-spacing-size1 .controls-manager, .just-perfection-api-controls-manager-spacing-size1 .secondary-monitor-workspaces { spacing: 1px; }
.just-perfection-api-controls-manager-spacing-size2 .controls-manager, .just-perfection-api-controls-manager-spacing-size2 .secondary-monitor-workspaces { spacing: 2px; }
.just-perfection-api-controls-manager-spacing-size3 .controls-manager, .just-perfection-api-controls-manager-spacing-size3 .secondary-monitor-workspaces { spacing: 3px; }
.just-perfection-api-controls-manager-spacing-size4 .controls-manager, .just-perfection-api-controls-manager-spacing-size4 .secondary-monitor-workspaces { spacing: 4px; }
.just-perfection-api-controls-manager-spacing-size5 .controls-manager, .just-perfection-api-controls-manager-spacing-size5 .secondary-monitor-workspaces { spacing: 5px; }
.just-perfection-api-controls-manager-spacing-size6 .controls-manager, .just-perfection-api-controls-manager-spacing-size6 .secondary-monitor-workspaces { spacing: 6px; }
.just-perfection-api-controls-manager-spacing-size7 .controls-manager, .just-perfection-api-controls-manager-spacing-size7 .secondary-monitor-workspaces { spacing: 7px; }
.just-perfection-api-controls-manager-spacing-size8 .controls-manager, .just-perfection-api-controls-manager-spacing-size8 .secondary-monitor-workspaces { spacing: 8px; }
.just-perfection-api-controls-manager-spacing-size9 .controls-manager, .just-perfection-api-controls-manager-spacing-size9 .secondary-monitor-workspaces { spacing: 9px; }
.just-perfection-api-controls-manager-spacing-size10 .controls-manager, .just-perfection-api-controls-manager-spacing-size10 .secondary-monitor-workspaces { spacing: 10px; }
.just-perfection-api-controls-manager-spacing-size11 .controls-manager, .just-perfection-api-controls-manager-spacing-size11 .secondary-monitor-workspaces { spacing: 11px; }
.just-perfection-api-controls-manager-spacing-size12 .controls-manager, .just-perfection-api-controls-manager-spacing-size12 .secondary-monitor-workspaces { spacing: 12px; }
.just-perfection-api-controls-manager-spacing-size13 .controls-manager, .just-perfection-api-controls-manager-spacing-size13 .secondary-monitor-workspaces { spacing: 13px; }
.just-perfection-api-controls-manager-spacing-size14 .controls-manager, .just-perfection-api-controls-manager-spacing-size14 .secondary-monitor-workspaces { spacing: 14px; }
.just-perfection-api-controls-manager-spacing-size15 .controls-manager, .just-perfection-api-controls-manager-spacing-size15 .secondary-monitor-workspaces { spacing: 15px; }
.just-perfection-api-controls-manager-spacing-size16 .controls-manager, .just-perfection-api-controls-manager-spacing-size16 .secondary-monitor-workspaces { spacing: 16px; }
.just-perfection-api-controls-manager-spacing-size17 .controls-manager, .just-perfection-api-controls-manager-spacing-size17 .secondary-monitor-workspaces { spacing: 17px; }
.just-perfection-api-controls-manager-spacing-size18 .controls-manager, .just-perfection-api-controls-manager-spacing-size18 .secondary-monitor-workspaces { spacing: 18px; }
.just-perfection-api-controls-manager-spacing-size19 .controls-manager, .just-perfection-api-controls-manager-spacing-size19 .secondary-monitor-workspaces { spacing: 19px; }
.just-perfection-api-controls-manager-spacing-size20 .controls-manager, .just-perfection-api-controls-manager-spacing-size20 .secondary-monitor-workspaces { spacing: 20px; }
.just-perfection-api-controls-manager-spacing-size21 .controls-manager, .just-perfection-api-controls-manager-spacing-size21 .secondary-monitor-workspaces { spacing: 21px; }
.just-perfection-api-controls-manager-spacing-size22 .controls-manager, .just-perfection-api-controls-manager-spacing-size22 .secondary-monitor-workspaces { spacing: 22px; }
.just-perfection-api-controls-manager-spacing-size23 .controls-manager, .just-perfection-api-controls-manager-spacing-size23 .secondary-monitor-workspaces { spacing: 23px; }
.just-perfection-api-controls-manager-spacing-size24 .controls-manager, .just-perfection-api-controls-manager-spacing-size24 .secondary-monitor-workspaces { spacing: 24px; }
.just-perfection-api-controls-manager-spacing-size25 .controls-manager, .just-perfection-api-controls-manager-spacing-size25 .secondary-monitor-workspaces { spacing: 25px; }
.just-perfection-api-controls-manager-spacing-size26 .controls-manager, .just-perfection-api-controls-manager-spacing-size26 .secondary-monitor-workspaces { spacing: 26px; }
.just-perfection-api-controls-manager-spacing-size27 .controls-manager, .just-perfection-api-controls-manager-spacing-size27 .secondary-monitor-workspaces { spacing: 27px; }
.just-perfection-api-controls-manager-spacing-size28 .controls-manager, .just-perfection-api-controls-manager-spacing-size28 .secondary-monitor-workspaces { spacing: 28px; }
.just-perfection-api-controls-manager-spacing-size29 .controls-manager, .just-perfection-api-controls-manager-spacing-size29 .secondary-monitor-workspaces { spacing: 29px; }
.just-perfection-api-controls-manager-spacing-size30 .controls-manager, .just-perfection-api-controls-manager-spacing-size30 .secondary-monitor-workspaces { spacing: 30px; }
.just-perfection-api-controls-manager-spacing-size31 .controls-manager, .just-perfection-api-controls-manager-spacing-size31 .secondary-monitor-workspaces { spacing: 31px; }
.just-perfection-api-controls-manager-spacing-size32 .controls-manager, .just-perfection-api-controls-manager-spacing-size32 .secondary-monitor-workspaces { spacing: 32px; }
.just-perfection-api-controls-manager-spacing-size33 .controls-manager, .just-perfection-api-controls-manager-spacing-size33 .secondary-monitor-workspaces { spacing: 33px; }
.just-perfection-api-controls-manager-spacing-size34 .controls-manager, .just-perfection-api-controls-manager-spacing-size34 .secondary-monitor-workspaces { spacing: 34px; }
.just-perfection-api-controls-manager-spacing-size35 .controls-manager, .just-perfection-api-controls-manager-spacing-size35 .secondary-monitor-workspaces { spacing: 35px; }
.just-perfection-api-controls-manager-spacing-size36 .controls-manager, .just-perfection-api-controls-manager-spacing-size36 .secondary-monitor-workspaces { spacing: 36px; }
.just-perfection-api-controls-manager-spacing-size37 .controls-manager, .just-perfection-api-controls-manager-spacing-size37 .secondary-monitor-workspaces { spacing: 37px; }
.just-perfection-api-controls-manager-spacing-size38 .controls-manager, .just-perfection-api-controls-manager-spacing-size38 .secondary-monitor-workspaces { spacing: 38px; }
.just-perfection-api-controls-manager-spacing-size39 .controls-manager, .just-perfection-api-controls-manager-spacing-size39 .secondary-monitor-workspaces { spacing: 39px; }
.just-perfection-api-controls-manager-spacing-size40 .controls-manager, .just-perfection-api-controls-manager-spacing-size40 .secondary-monitor-workspaces { spacing: 40px; }
.just-perfection-api-controls-manager-spacing-size41 .controls-manager, .just-perfection-api-controls-manager-spacing-size41 .secondary-monitor-workspaces { spacing: 41px; }
.just-perfection-api-controls-manager-spacing-size42 .controls-manager, .just-perfection-api-controls-manager-spacing-size42 .secondary-monitor-workspaces { spacing: 42px; }
.just-perfection-api-controls-manager-spacing-size43 .controls-manager, .just-perfection-api-controls-manager-spacing-size43 .secondary-monitor-workspaces { spacing: 43px; }
.just-perfection-api-controls-manager-spacing-size44 .controls-manager, .just-perfection-api-controls-manager-spacing-size44 .secondary-monitor-workspaces { spacing: 44px; }
.just-perfection-api-controls-manager-spacing-size45 .controls-manager, .just-perfection-api-controls-manager-spacing-size45 .secondary-monitor-workspaces { spacing: 45px; }
.just-perfection-api-controls-manager-spacing-size46 .controls-manager, .just-perfection-api-controls-manager-spacing-size46 .secondary-monitor-workspaces { spacing: 46px; }
.just-perfection-api-controls-manager-spacing-size47 .controls-manager, .just-perfection-api-controls-manager-spacing-size47 .secondary-monitor-workspaces { spacing: 47px; }
.just-perfection-api-controls-manager-spacing-size48 .controls-manager, .just-perfection-api-controls-manager-spacing-size48 .secondary-monitor-workspaces { spacing: 48px; }
.just-perfection-api-controls-manager-spacing-size49 .controls-manager, .just-perfection-api-controls-manager-spacing-size49 .secondary-monitor-workspaces { spacing: 49px; }
.just-perfection-api-controls-manager-spacing-size50 .controls-manager, .just-perfection-api-controls-manager-spacing-size50 .secondary-monitor-workspaces { spacing: 50px; }
.just-perfection-api-controls-manager-spacing-size51 .controls-manager, .just-perfection-api-controls-manager-spacing-size51 .secondary-monitor-workspaces { spacing: 51px; }
.just-perfection-api-controls-manager-spacing-size52 .controls-manager, .just-perfection-api-controls-manager-spacing-size52 .secondary-monitor-workspaces { spacing: 52px; }
.just-perfection-api-controls-manager-spacing-size53 .controls-manager, .just-perfection-api-controls-manager-spacing-size53 .secondary-monitor-workspaces { spacing: 53px; }
.just-perfection-api-controls-manager-spacing-size54 .controls-manager, .just-perfection-api-controls-manager-spacing-size54 .secondary-monitor-workspaces { spacing: 54px; }
.just-perfection-api-controls-manager-spacing-size55 .controls-manager, .just-perfection-api-controls-manager-spacing-size55 .secondary-monitor-workspaces { spacing: 55px; }
.just-perfection-api-controls-manager-spacing-size56 .controls-manager, .just-perfection-api-controls-manager-spacing-size56 .secondary-monitor-workspaces { spacing: 56px; }
.just-perfection-api-controls-manager-spacing-size57 .controls-manager, .just-perfection-api-controls-manager-spacing-size57 .secondary-monitor-workspaces { spacing: 57px; }
.just-perfection-api-controls-manager-spacing-size58 .controls-manager, .just-perfection-api-controls-manager-spacing-size58 .secondary-monitor-workspaces { spacing: 58px; }
.just-perfection-api-controls-manager-spacing-size59 .controls-manager, .just-perfection-api-controls-manager-spacing-size59 .secondary-monitor-workspaces { spacing: 59px; }
.just-perfection-api-controls-manager-spacing-size60 .controls-manager, .just-perfection-api-controls-manager-spacing-size60 .secondary-monitor-workspaces { spacing: 60px; }
.just-perfection-api-controls-manager-spacing-size61 .controls-manager, .just-perfection-api-controls-manager-spacing-size61 .secondary-monitor-workspaces { spacing: 61px; }
.just-perfection-api-controls-manager-spacing-size62 .controls-manager, .just-perfection-api-controls-manager-spacing-size62 .secondary-monitor-workspaces { spacing: 62px; }
.just-perfection-api-controls-manager-spacing-size63 .controls-manager, .just-perfection-api-controls-manager-spacing-size63 .secondary-monitor-workspaces { spacing: 63px; }
.just-perfection-api-controls-manager-spacing-size64 .controls-manager, .just-perfection-api-controls-manager-spacing-size64 .secondary-monitor-workspaces { spacing: 64px; }
.just-perfection-api-controls-manager-spacing-size65 .controls-manager, .just-perfection-api-controls-manager-spacing-size65 .secondary-monitor-workspaces { spacing: 65px; }
.just-perfection-api-controls-manager-spacing-size66 .controls-manager, .just-perfection-api-controls-manager-spacing-size66 .secondary-monitor-workspaces { spacing: 66px; }
.just-perfection-api-controls-manager-spacing-size67 .controls-manager, .just-perfection-api-controls-manager-spacing-size67 .secondary-monitor-workspaces { spacing: 67px; }
.just-perfection-api-controls-manager-spacing-size68 .controls-manager, .just-perfection-api-controls-manager-spacing-size68 .secondary-monitor-workspaces { spacing: 68px; }
.just-perfection-api-controls-manager-spacing-size69 .controls-manager, .just-perfection-api-controls-manager-spacing-size69 .secondary-monitor-workspaces { spacing: 69px; }
.just-perfection-api-controls-manager-spacing-size70 .controls-manager, .just-perfection-api-controls-manager-spacing-size70 .secondary-monitor-workspaces { spacing: 70px; }
.just-perfection-api-controls-manager-spacing-size71 .controls-manager, .just-perfection-api-controls-manager-spacing-size71 .secondary-monitor-workspaces { spacing: 71px; }
.just-perfection-api-controls-manager-spacing-size72 .controls-manager, .just-perfection-api-controls-manager-spacing-size72 .secondary-monitor-workspaces { spacing: 72px; }
.just-perfection-api-controls-manager-spacing-size73 .controls-manager, .just-perfection-api-controls-manager-spacing-size73 .secondary-monitor-workspaces { spacing: 73px; }
.just-perfection-api-controls-manager-spacing-size74 .controls-manager, .just-perfection-api-controls-manager-spacing-size74 .secondary-monitor-workspaces { spacing: 74px; }
.just-perfection-api-controls-manager-spacing-size75 .controls-manager, .just-perfection-api-controls-manager-spacing-size75 .secondary-monitor-workspaces { spacing: 75px; }
.just-perfection-api-controls-manager-spacing-size76 .controls-manager, .just-perfection-api-controls-manager-spacing-size76 .secondary-monitor-workspaces { spacing: 76px; }
.just-perfection-api-controls-manager-spacing-size77 .controls-manager, .just-perfection-api-controls-manager-spacing-size77 .secondary-monitor-workspaces { spacing: 77px; }
.just-perfection-api-controls-manager-spacing-size78 .controls-manager, .just-perfection-api-controls-manager-spacing-size78 .secondary-monitor-workspaces { spacing: 78px; }
.just-perfection-api-controls-manager-spacing-size79 .controls-manager, .just-perfection-api-controls-manager-spacing-size79 .secondary-monitor-workspaces { spacing: 79px; }
.just-perfection-api-controls-manager-spacing-size80 .controls-manager, .just-perfection-api-controls-manager-spacing-size80 .secondary-monitor-workspaces { spacing: 80px; }
.just-perfection-api-controls-manager-spacing-size81 .controls-manager, .just-perfection-api-controls-manager-spacing-size81 .secondary-monitor-workspaces { spacing: 81px; }
.just-perfection-api-controls-manager-spacing-size82 .controls-manager, .just-perfection-api-controls-manager-spacing-size82 .secondary-monitor-workspaces { spacing: 82px; }
.just-perfection-api-controls-manager-spacing-size83 .controls-manager, .just-perfection-api-controls-manager-spacing-size83 .secondary-monitor-workspaces { spacing: 83px; }
.just-perfection-api-controls-manager-spacing-size84 .controls-manager, .just-perfection-api-controls-manager-spacing-size84 .secondary-monitor-workspaces { spacing: 84px; }
.just-perfection-api-controls-manager-spacing-size85 .controls-manager, .just-perfection-api-controls-manager-spacing-size85 .secondary-monitor-workspaces { spacing: 85px; }
.just-perfection-api-controls-manager-spacing-size86 .controls-manager, .just-perfection-api-controls-manager-spacing-size86 .secondary-monitor-workspaces { spacing: 86px; }
.just-perfection-api-controls-manager-spacing-size87 .controls-manager, .just-perfection-api-controls-manager-spacing-size87 .secondary-monitor-workspaces { spacing: 87px; }
.just-perfection-api-controls-manager-spacing-size88 .controls-manager, .just-perfection-api-controls-manager-spacing-size88 .secondary-monitor-workspaces { spacing: 88px; }
.just-perfection-api-controls-manager-spacing-size89 .controls-manager, .just-perfection-api-controls-manager-spacing-size89 .secondary-monitor-workspaces { spacing: 89px; }
.just-perfection-api-controls-manager-spacing-size90 .controls-manager, .just-perfection-api-controls-manager-spacing-size90 .secondary-monitor-workspaces { spacing: 90px; }
.just-perfection-api-controls-manager-spacing-size91 .controls-manager, .just-perfection-api-controls-manager-spacing-size91 .secondary-monitor-workspaces { spacing: 91px; }
.just-perfection-api-controls-manager-spacing-size92 .controls-manager, .just-perfection-api-controls-manager-spacing-size92 .secondary-monitor-workspaces { spacing: 92px; }
.just-perfection-api-controls-manager-spacing-size93 .controls-manager, .just-perfection-api-controls-manager-spacing-size93 .secondary-monitor-workspaces { spacing: 93px; }
.just-perfection-api-controls-manager-spacing-size94 .controls-manager, .just-perfection-api-controls-manager-spacing-size94 .secondary-monitor-workspaces { spacing: 94px; }
.just-perfection-api-controls-manager-spacing-size95 .controls-manager, .just-perfection-api-controls-manager-spacing-size95 .secondary-monitor-workspaces { spacing: 95px; }
.just-perfection-api-controls-manager-spacing-size96 .controls-manager, .just-perfection-api-controls-manager-spacing-size96 .secondary-monitor-workspaces { spacing: 96px; }
.just-perfection-api-controls-manager-spacing-size97 .controls-manager, .just-perfection-api-controls-manager-spacing-size97 .secondary-monitor-workspaces { spacing: 97px; }
.just-perfection-api-controls-manager-spacing-size98 .controls-manager, .just-perfection-api-controls-manager-spacing-size98 .secondary-monitor-workspaces { spacing: 98px; }
.just-perfection-api-controls-manager-spacing-size99 .controls-manager, .just-perfection-api-controls-manager-spacing-size99 .secondary-monitor-workspaces { spacing: 99px; }
.just-perfection-api-controls-manager-spacing-size100 .controls-manager, .just-perfection-api-controls-manager-spacing-size100 .secondary-monitor-workspaces { spacing: 100px; }
.just-perfection-api-controls-manager-spacing-size101 .controls-manager, .just-perfection-api-controls-manager-spacing-size101 .secondary-monitor-workspaces { spacing: 101px; }
.just-perfection-api-controls-manager-spacing-size102 .controls-manager, .just-perfection-api-controls-manager-spacing-size102 .secondary-monitor-workspaces { spacing: 102px; }
.just-perfection-api-controls-manager-spacing-size103 .controls-manager, .just-perfection-api-controls-manager-spacing-size103 .secondary-monitor-workspaces { spacing: 103px; }
.just-perfection-api-controls-manager-spacing-size104 .controls-manager, .just-perfection-api-controls-manager-spacing-size104 .secondary-monitor-workspaces { spacing: 104px; }
.just-perfection-api-controls-manager-spacing-size105 .controls-manager, .just-perfection-api-controls-manager-spacing-size105 .secondary-monitor-workspaces { spacing: 105px; }
.just-perfection-api-controls-manager-spacing-size106 .controls-manager, .just-perfection-api-controls-manager-spacing-size106 .secondary-monitor-workspaces { spacing: 106px; }
.just-perfection-api-controls-manager-spacing-size107 .controls-manager, .just-perfection-api-controls-manager-spacing-size107 .secondary-monitor-workspaces { spacing: 107px; }
.just-perfection-api-controls-manager-spacing-size108 .controls-manager, .just-perfection-api-controls-manager-spacing-size108 .secondary-monitor-workspaces { spacing: 108px; }
.just-perfection-api-controls-manager-spacing-size109 .controls-manager, .just-perfection-api-controls-manager-spacing-size109 .secondary-monitor-workspaces { spacing: 109px; }
.just-perfection-api-controls-manager-spacing-size110 .controls-manager, .just-perfection-api-controls-manager-spacing-size110 .secondary-monitor-workspaces { spacing: 110px; }
.just-perfection-api-controls-manager-spacing-size111 .controls-manager, .just-perfection-api-controls-manager-spacing-size111 .secondary-monitor-workspaces { spacing: 111px; }
.just-perfection-api-controls-manager-spacing-size112 .controls-manager, .just-perfection-api-controls-manager-spacing-size112 .secondary-monitor-workspaces { spacing: 112px; }
.just-perfection-api-controls-manager-spacing-size113 .controls-manager, .just-perfection-api-controls-manager-spacing-size113 .secondary-monitor-workspaces { spacing: 113px; }
.just-perfection-api-controls-manager-spacing-size114 .controls-manager, .just-perfection-api-controls-manager-spacing-size114 .secondary-monitor-workspaces { spacing: 114px; }
.just-perfection-api-controls-manager-spacing-size115 .controls-manager, .just-perfection-api-controls-manager-spacing-size115 .secondary-monitor-workspaces { spacing: 115px; }
.just-perfection-api-controls-manager-spacing-size116 .controls-manager, .just-perfection-api-controls-manager-spacing-size116 .secondary-monitor-workspaces { spacing: 116px; }
.just-perfection-api-controls-manager-spacing-size117 .controls-manager, .just-perfection-api-controls-manager-spacing-size117 .secondary-monitor-workspaces { spacing: 117px; }
.just-perfection-api-controls-manager-spacing-size118 .controls-manager, .just-perfection-api-controls-manager-spacing-size118 .secondary-monitor-workspaces { spacing: 118px; }
.just-perfection-api-controls-manager-spacing-size119 .controls-manager, .just-perfection-api-controls-manager-spacing-size119 .secondary-monitor-workspaces { spacing: 119px; }
.just-perfection-api-controls-manager-spacing-size120 .controls-manager, .just-perfection-api-controls-manager-spacing-size120 .secondary-monitor-workspaces { spacing: 120px; }
.just-perfection-api-controls-manager-spacing-size121 .controls-manager, .just-perfection-api-controls-manager-spacing-size121 .secondary-monitor-workspaces { spacing: 121px; }
.just-perfection-api-controls-manager-spacing-size122 .controls-manager, .just-perfection-api-controls-manager-spacing-size122 .secondary-monitor-workspaces { spacing: 122px; }
.just-perfection-api-controls-manager-spacing-size123 .controls-manager, .just-perfection-api-controls-manager-spacing-size123 .secondary-monitor-workspaces { spacing: 123px; }
.just-perfection-api-controls-manager-spacing-size124 .controls-manager, .just-perfection-api-controls-manager-spacing-size124 .secondary-monitor-workspaces { spacing: 124px; }
.just-perfection-api-controls-manager-spacing-size125 .controls-manager, .just-perfection-api-controls-manager-spacing-size125 .secondary-monitor-workspaces { spacing: 125px; }
.just-perfection-api-controls-manager-spacing-size126 .controls-manager, .just-perfection-api-controls-manager-spacing-size126 .secondary-monitor-workspaces { spacing: 126px; }
.just-perfection-api-controls-manager-spacing-size127 .controls-manager, .just-perfection-api-controls-manager-spacing-size127 .secondary-monitor-workspaces { spacing: 127px; }
.just-perfection-api-controls-manager-spacing-size128 .controls-manager, .just-perfection-api-controls-manager-spacing-size128 .secondary-monitor-workspaces { spacing: 128px; }
.just-perfection-api-controls-manager-spacing-size129 .controls-manager, .just-perfection-api-controls-manager-spacing-size129 .secondary-monitor-workspaces { spacing: 129px; }
.just-perfection-api-controls-manager-spacing-size130 .controls-manager, .just-perfection-api-controls-manager-spacing-size130 .secondary-monitor-workspaces { spacing: 130px; }
.just-perfection-api-controls-manager-spacing-size131 .controls-manager, .just-perfection-api-controls-manager-spacing-size131 .secondary-monitor-workspaces { spacing: 131px; }
.just-perfection-api-controls-manager-spacing-size132 .controls-manager, .just-perfection-api-controls-manager-spacing-size132 .secondary-monitor-workspaces { spacing: 132px; }
.just-perfection-api-controls-manager-spacing-size133 .controls-manager, .just-perfection-api-controls-manager-spacing-size133 .secondary-monitor-workspaces { spacing: 133px; }
.just-perfection-api-controls-manager-spacing-size134 .controls-manager, .just-perfection-api-controls-manager-spacing-size134 .secondary-monitor-workspaces { spacing: 134px; }
.just-perfection-api-controls-manager-spacing-size135 .controls-manager, .just-perfection-api-controls-manager-spacing-size135 .secondary-monitor-workspaces { spacing: 135px; }
.just-perfection-api-controls-manager-spacing-size136 .controls-manager, .just-perfection-api-controls-manager-spacing-size136 .secondary-monitor-workspaces { spacing: 136px; }
.just-perfection-api-controls-manager-spacing-size137 .controls-manager, .just-perfection-api-controls-manager-spacing-size137 .secondary-monitor-workspaces { spacing: 137px; }
.just-perfection-api-controls-manager-spacing-size138 .controls-manager, .just-perfection-api-controls-manager-spacing-size138 .secondary-monitor-workspaces { spacing: 138px; }
.just-perfection-api-controls-manager-spacing-size139 .controls-manager, .just-perfection-api-controls-manager-spacing-size139 .secondary-monitor-workspaces { spacing: 139px; }
.just-perfection-api-controls-manager-spacing-size140 .controls-manager, .just-perfection-api-controls-manager-spacing-size140 .secondary-monitor-workspaces { spacing: 140px; }
.just-perfection-api-controls-manager-spacing-size141 .controls-manager, .just-perfection-api-controls-manager-spacing-size141 .secondary-monitor-workspaces { spacing: 141px; }
.just-perfection-api-controls-manager-spacing-size142 .controls-manager, .just-perfection-api-controls-manager-spacing-size142 .secondary-monitor-workspaces { spacing: 142px; }
.just-perfection-api-controls-manager-spacing-size143 .controls-manager, .just-perfection-api-controls-manager-spacing-size143 .secondary-monitor-workspaces { spacing: 143px; }
.just-perfection-api-controls-manager-spacing-size144 .controls-manager, .just-perfection-api-controls-manager-spacing-size144 .secondary-monitor-workspaces { spacing: 144px; }
.just-perfection-api-controls-manager-spacing-size145 .controls-manager, .just-perfection-api-controls-manager-spacing-size145 .secondary-monitor-workspaces { spacing: 145px; }
.just-perfection-api-controls-manager-spacing-size146 .controls-manager, .just-perfection-api-controls-manager-spacing-size146 .secondary-monitor-workspaces { spacing: 146px; }
.just-perfection-api-controls-manager-spacing-size147 .controls-manager, .just-perfection-api-controls-manager-spacing-size147 .secondary-monitor-workspaces { spacing: 147px; }
.just-perfection-api-controls-manager-spacing-size148 .controls-manager, .just-perfection-api-controls-manager-spacing-size148 .secondary-monitor-workspaces { spacing: 148px; }
.just-perfection-api-controls-manager-spacing-size149 .controls-manager, .just-perfection-api-controls-manager-spacing-size149 .secondary-monitor-workspaces { spacing: 149px; }
.just-perfection-api-controls-manager-spacing-size150 .controls-manager, .just-perfection-api-controls-manager-spacing-size150 .secondary-monitor-workspaces { spacing: 150px; }

.just-perfection-api-osd-position-top .osd-window { margin: 4em 2.5em; }
.just-perfection-api-osd-position-center .osd-window { margin: 4em 3.5em; }
.just-perfection-api-osd-position-bottom .osd-window { margin: 4em 3em; }

.just-perfection-api-no-dash-separator .dash-separator,
.just-perfection-api-no-dash-app-running-dot #dash .app-well-app-running-dot,
.just-perfection-api-no-dash-app-running-dot #dash .app-grid-running-dot
{
    width: 0;
    height: 0;
    spacing: 0;
    padding: 0;
    margin: 0;
}

.just-perfection-api-no-workspaces-in-app-grid .workspace-background
{
    background-color: transparent;
    box-shadow: 0 4px 16px 4px transparent;
}

.just-perfection-api-no-power-icon .power-status StIcon
{
    width: 0;
    height: 0;
    padding: 0;
}
